Crate c_ares_sys [−] [src]
Low-level bindings for the c-ares library.
In most cases this crate should not be used directly. The c-ares crate provides a safe wrapper and should be preferred wherever possible.
Structs
Constants
Functions
Type Definitions
| ares_callback | |
| ares_channel | |
| ares_host_callback | |
| ares_nameinfo_callback | |
| ares_sock_config_callback | |
| ares_sock_create_callback | |
| ares_sock_state_cb | |
| ares_socket_t | |
| ares_socklen_t | |
| ares_ssize_t |
Unions
| ares_addr_node__bindgen_ty_1 | |
| ares_addr_port_node__bindgen_ty_1 | |
| ares_in6_addr__bindgen_ty_1 |